/**
 *   .       .     ..
 *  _| _  _.*|_  _ ||
 * (_](/,(_.|[_)(/,||
 *
 * DeciBell : A Java Tool for creating and managing relational databases.
 *  DeciBell is a Object - Relation database mapper for java applications providing
 * an easy-to-use interface making it easy for the developer to build a relational
 * database and moreover perform database operations easily!
 *  This project was developed at the Automatic Control Lab in the Chemical Engineering
 * School of the National Technical University of Athens. Please read README for more
 * information.
 *
 * Copyright (C) 2009-2010 Charalampos Chomenides & Pantelis Sopasakis
 *                         kinkyDesign ~ OpenSource Development

 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact:
 * hampos Att yahoo d0t c0m
 * chvng att mail D0t ntua dd0T gr
 * Address: Iroon Politechniou St. 9, Zografou, Athens Greece
 * tel. +30 210 7723236
 */
package org.kinkydesign.decibell.db.derby;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import org.kinkydesign.decibell.collections.LogicalOperator;
import org.kinkydesign.decibell.collections.Qualifier;
import org.kinkydesign.decibell.collections.SQLType;
import org.kinkydesign.decibell.db.Table;
import org.kinkydesign.decibell.db.interfaces.JTableColumn;

import static org.kinkydesign.decibell.db.derby.util.DerbyKeyWord.*;

/**
 *
 * <p  align="justify" style="width:60%">
 * A Table in the database is characterized by its columns. This class offers a
 * flexible tool for manipulating database tables (creating and deleting them).
 * </p>
 *
 * @author Pantelis Sopasakis
 * @author Charalampos Chomenides
 */
public class DerbyTable extends Table {

    /**
     * 
     * Construct a new Table object.
     */
    public DerbyTable() {
        super();
    }

    /**
     * Constructs the Derby SQL command that creates the required table according
     * to the TableColumns it includes.
     * @return SQL Table Creation String for Derby
     */
    public String getCreationSQL() 
    {
        StringBuffer bufferSQL = new StringBuffer();
        bufferSQL.append(CREATE_TABLE + SPACE + getTableName() + NEWLINE + LEFT_PAR + SPACE);
        Iterator<JTableColumn> it = super.getTableColumns().iterator();
        while (it.hasNext()) {
            JTableColumn column = it.next();
            bufferSQL.append(column.getColumnName() + SPACE + column.getColumnType().toString() + SPACE);

            if (column.isUnique()) {
                bufferSQL.append(UNIQUE + SPACE);
            }
            if (column.isNotNull() || column.isPrimaryKey()) {
                bufferSQL.append(NOT_NULL + SPACE);
            }
            if (column.isAutoGenerated()) {
                bufferSQL.append(AUTO_GEN + SPACE);
            }

            if (column.hasDefault()) {
                String DEFAULT_VALUE = "";
                if (column.isTypeNumeric()) {
                    DEFAULT_VALUE = column.getDefaultValue();
                } else {
                    DEFAULT_VALUE = SINGLE_QUOTE + column.getDefaultValue() + SINGLE_QUOTE;
                }
                bufferSQL.append(DEFAULT + SPACE + DEFAULT_VALUE + SPACE);
            }

            if (column.isConstrained()) {
                bufferSQL.append(getConstraint(column) + SPACE);
            }
            if (it.hasNext()) {
                bufferSQL.append(COMMA + NEWLINE);
            }
        }

        if (!getPrimaryKeyColumns().isEmpty()) {
            bufferSQL.append(COMMA + NEWLINE + Pk + SPACE + LEFT_PAR);
            it = getPrimaryKeyColumns().iterator();
            while (it.hasNext()) {
                bufferSQL.append(it.next().getColumnName());
                if (it.hasNext()) {
                    bufferSQL.append(COMMA);
                }
            }
            bufferSQL.append(RIGHT_PAR);
        }

        
        Set<Set<JTableColumn>> foreignGroups = getForeignColumnsByGroup();
        if (!foreignGroups.isEmpty()) {
            bufferSQL.append(COMMA + NEWLINE);
            Iterator<Set<JTableColumn>> setIt = foreignGroups.iterator();
            while (setIt.hasNext()) {
                Set<JTableColumn> group = setIt.next();
                String foreignKey = "";
                String references = "";
                String options = "";
                it = group.iterator();
                while (it.hasNext()) {
                    JTableColumn col = it.next();
                    if (foreignKey.isEmpty()) {
                        foreignKey = Fk + SPACE + LEFT_PAR;
                    }
                    if (references.isEmpty()) {
                        references = REFERENCES + SPACE + col.getReferenceTableName() + LEFT_PAR;
                    }
                    if (options.isEmpty()) {
                        options = ON + SPACE + DELETE + SPACE + col.getOnDelete() + SPACE + ON + SPACE + UPDATE + SPACE + col.getOnUpdate();
                    }
                    foreignKey += col.getColumnName();
                    references += col.getReferenceColumnName();
                    if (it.hasNext()) {
                        foreignKey += COMMA;
                        references += COMMA;
                    }
                }
                foreignKey += RIGHT_PAR + SPACE;
                references += RIGHT_PAR + SPACE;
                bufferSQL.append(foreignKey + references + options);
                foreignKey = "";
                references = "";
                options = "";
                if (setIt.hasNext()) {
                    bufferSQL.append(COMMA + NEWLINE);
                }
            }
        }

        
        bufferSQL.append(NEWLINE + RIGHT_PAR);

        return bufferSQL.toString();
    }

    /**
     * Constructs the Deletion SQL command required to delete this table.
     * @return SQL Table Deletion String for Derby
     */
    public String getDeletionSQL()
    {
        String SQL = DROP_TABLE + SPACE + getTableName();
        return SQL;
    }

    /**
     * Private Method that constructs the SQL String regarding a column's
     * Constraints.
     * @param col the specified TableColumn
     * @return SQL String describing Constraints
     */
    private String getConstraint(JTableColumn col) {
        StringBuffer constraint = new StringBuffer();
        String lowStr = "";
        String highStr = "";
        String domainStr = "";
        Set<String> tempSet = new HashSet<String>();
        if (col.isConstrained()) {
            if (col.hasLow()) {
                lowStr += col.getColumnName() + Qualifier.GREATER_EQUAL + col.getLow();
                tempSet.add(lowStr);
            }
            if (col.hasHigh()) {
                highStr += col.getColumnName() + Qualifier.LESS_EQUAL + col.getHigh();
                tempSet.add(highStr);
            }
            if (col.hasDomain()) {
                for (int i = 0; i < col.getDomain().length; i++) {
                    if (!col.getDomain()[i].isEmpty()) {
                        if (domainStr.isEmpty()) {
                            domainStr = col.getColumnName() + SPACE + IN + SPACE + LEFT_PAR;
                        } else {
                            domainStr += COMMA + SPACE;
                        }
                        if (col.getColumnType().equals(SQLType.VARCHAR) || col.getColumnType().equals(SQLType.LONG_VARCHAR)) {
                            domainStr += SINGLE_QUOTE + col.getDomain()[i] + SINGLE_QUOTE;
                        } else {
                            domainStr += col.getDomain()[i];
                        }
                        if (i == col.getDomain().length - 1) {
                            domainStr += SPACE + RIGHT_PAR;
                        }
                    }
                }
                tempSet.add(domainStr);
            }
            Iterator<String> it = tempSet.iterator();
            while (it.hasNext()) {
                if (constraint.toString().isEmpty()) {
                    constraint.append(SPACE + CONSTRAINT + SPACE + col.getColumnName() + UNDERSCORE + CONSTRAINT + SPACE + CHECK + SPACE + LEFT_PAR);
                    constraint.append(it.next());
                } else {
                    constraint.append(SPACE + LogicalOperator.AND + SPACE + it.next());
                }
                if (!it.hasNext()) {
                    constraint.append(SPACE + RIGHT_PAR);
                }
            }
        }
        return constraint.toString();
    }

    public boolean hasSelfReferences()
    {
        return getSelfReferences().size() == 0 ? false : true;
    }

    public List<JTableColumn> getSelfReferences()
    {
        List<JTableColumn> selfReferences = new LinkedList<JTableColumn>();
        for (JTableColumn tc : getForeignKeyColumns()) {
            if (tc.getReferenceTable().equals(this)) {
                selfReferences.add(tc);
            }
        }
        return selfReferences;
    }

    
}
